#include <stdio.h>
#include <stdlib.h>
#include "matrix_operation.h"
#include "sparse_matrix.h"
#include "read_from_mtx.h"
#include "vector.h"
#include "power_algorithm.h"
#include "gauss_seidel.h"
#include "time_helper.h"
#include <sys/time.h>

void test_stationary_distribution(const char *path) {
    struct timeval tvstart, tvend;

    gettimeofday(&tvstart, NULL);
    SparseMatrix *matrix = read_sparse_matrix_from_mtx(path);
    convert_to_stochastic(matrix);
    gettimeofday(&tvend, NULL);
    print_time_diff("Read and convert matrix", &tvstart, &tvend);

    double epsilon = 1e-10;
    double alpha = 0.8;

    gettimeofday(&tvstart, NULL);
    double *pi_power = pagerank(matrix, epsilon, alpha);
    gettimeofday(&tvend, NULL);
    print_time_diff("Pagerank", &tvstart, &tvend);

    gettimeofday(&tvstart, NULL);
    double *pi_new = gauss_seidel_pagerank(matrix, epsilon, alpha);

    gettimeofday(&tvend, NULL);
    print_time_diff("Gauss-Seidel:", &tvstart, &tvend);

    double diff = diff_norm_vector(pi_power, pi_new, matrix->num_nodes);
    printf("Difference norm Pagerank and Gauss-Seidel: %.16f\n", diff);

    free(pi_power);
    free(pi_new);
    free_sparse_matrix(matrix);
}

int main() {
    test_stationary_distribution("./out/input.mtx");
    return 0;
}
